

package Mail::SpamAssassin::Plugin::WhiteListSubject;

use Mail::SpamAssassin::Plugin;
use strict;
use warnings;
use re 'taint';

our @ISA = qw(Mail::SpamAssassin::Plugin);

sub new {
  my $class = shift;
  my $mailsaobject = shift;

  $class = ref($class) || $class;
  my $self = $class->SUPER::new($mailsaobject);
  bless ($self, $class);

  $self->register_eval_rule ("check_subject_in_whitelist");
  $self->register_eval_rule ("check_subject_in_blacklist");

  $self->set_config($mailsaobject->{conf});

  return $self;
}

sub set_config {
  my ($self, $conf) = @_;

  my @cmds;

  push(@cmds, {
	       setting => 'whitelist_subject',
	       default => {},
               type => $Mail::SpamAssassin::Conf::CONF_TYPE_ADDRLIST,
	       code => sub {
		 my ($self, $key, $value, $line) = @_;

		 $value = lc $value;
		 my $re = $value;
		 $re =~ s/[\000\\\(]/_/gs;                   # paranoia
		 $re =~ s/([^\*\?_a-zA-Z0-9])/\\$1/g;        # escape any possible metachars
		 $re =~ tr/?/./;                             # "?" -> "."
                 $re =~ s/\*+/\.\*/g;                        # "*" -> "any string"
		 $conf->{$key}->{$value} = ${re};
	       }});

  push(@cmds, {
	       setting => 'blacklist_subject',
	       default => {},
               type => $Mail::SpamAssassin::Conf::CONF_TYPE_ADDRLIST,
	       code => sub {
		 my ($self, $key, $value, $line) = @_;

		 $value = lc $value;
		 my $re = $value;
		 $re =~ s/[\000\\\(]/_/gs;                   # paranoia
		 $re =~ s/([^\*\?_a-zA-Z0-9])/\\$1/g;        # escape any possible metachars
		 $re =~ tr/?/./;                             # "?" -> "."
                 $re =~ s/\*+/\.\*/g;                        # "*" -> "any string"
		 $conf->{$key}->{$value} = ${re};
	       }});

  $conf->{parser}->register_commands(\@cmds);
}

sub check_subject_in_whitelist {
  my ($self, $permsgstatus) = @_;

  my $subject = $permsgstatus->get('Subject');

  return 0 unless $subject ne '';

  return $self->_check_subject($permsgstatus->{conf}->{whitelist_subject}, $subject);
}

sub check_subject_in_blacklist {
  my ($self, $permsgstatus) = @_;

  my $subject = $permsgstatus->get('Subject');

  return 0 unless $subject ne '';

  return $self->_check_subject($permsgstatus->{conf}->{blacklist_subject}, $subject);
}

sub _check_subject {
  my ($self, $list, $subject) = @_;

  $subject = lc $subject;

  return 1 if defined($list->{$subject});

  study $subject;  # study is a no-op since perl 5.16.0, eliminating bugs
  foreach my $regexp (values %{$list}) {
    if ($subject =~ qr/$regexp/i) {
      return 1;
    }
  }

  return 0;
}

1;
